﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;

namespace SushiBarBusinessLogic.BusinessLogics
{
    public class SushiLogic : ISushiLogic
    {
        private readonly ILogger _logger;
        private readonly ISushiStorage _sushiStorage;
        public SushiLogic(ILogger<SushiLogic> logger, ISushiStorage sushiStorage)
        {
            _logger = logger;
            _sushiStorage = sushiStorage;
        }
        public List<SushiViewModel>? ReadList(SushiSearchModel? model)
        {
            _logger.LogInformation("ReadList. SushiName:{SushiName}. Id:{ Id}", model?.SushiName, model?.Id);
            var list = model == null ? _sushiStorage.GetFullList() :
                _sushiStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public SushiViewModel? ReadElement(SushiSearchModel model)
        {
            if(model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. SushiName:{SushiName}. Id:{ Id}", model.SushiName, model.Id);
            var element = _sushiStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            return element;
        }
        public bool Create(SushiBindingModel model)
        {
            CheckModel(model);
            if (_sushiStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(SushiBindingModel model)
        {
            CheckModel(model);
            if (_sushiStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(SushiBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_sushiStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(SushiBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.SushiName))
            {
                throw new ArgumentException("Нет названия продукта", nameof(model.SushiName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Цена продукта должна быть больше 0", nameof(model.Price));
            }
            var element = _sushiStorage.GetElement(new SushiSearchModel
            {
                SushiName = model.SushiName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Продукт с таким названием уже есть");
            }
        }
    }
}
